unit Unit1;

interface

uses
  mmsystem,Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs,binkw32, StdCtrls, ExtCtrls, ComCtrls,directsound, MPlayer;

type

  TMovieThread = class(Tthread)
  protected
    FSTream:TFileStream;
    Fbmp : TBitmap;
    Fmovie : hbink;
    Fcanvas:Tcanvas;
    Frealtimeinfo : TBinkRealTimeInfo;
    FGoto:boolean;
    FGotoframe:longword;
    procedure Execute; override;
    procedure draw;
    procedure Notify;
  public
    OnNotify : procedure(bink:Hbink;Info:TBinkRealTimeInfo)of object;
    Constructor Create(binkFile:string;Canvas:Tcanvas;offset:Longword=0;Mode:longword=0);
    Procedure Play;
    Procedure GotoFrame(Frame:longword);
  end;

  TForm1 = class(TForm)
    Panel1: TPanel;
    PaintBox1: TPaintBox;
    Edit1: TEdit;
    PlayButton: TButton;
    ComboBox1: TComboBox;
    StatusBar1: TStatusBar;
    ScrollBar1: TScrollBar;
    Button1: TButton;
    TrackBar1: TTrackBar;
    TrackBar2: TTrackBar;
    Edit2: TEdit;
    ComboBox2: TComboBox;
    Button2: TButton;
    procedure PlayButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ScrollBar1Scroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure Button1Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure TrackBar2Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    mt : TMovieThread;
    m:hbink;
    ISound  : IDirectSound;
    procedure FillInfo(bnk:hbink);
    Procedure ThreadNotify(bink:Hbink;Info:TBinkRealTimeInfo);
  end;



var
  Form1: TForm1;
  movie : Hbink;

implementation

uses Types;

{$R *.dfm}

{-------------------------------------------------------------------------------
    TMovieThread
    Takes a bink file and plays it!
-------------------------------------------------------------------------------}

constructor TMovieThread.Create;
begin
  Fstream := TFileStream.create(Binkfile,fmOpenRead);
  fstream.Position:=offset;
  FGoto:=false;
  mode := mode OR BINK_OPEN_STREAM; // tell bink that we have a filehandle!

  Fmovie := BinkOpen(Pointer(Fstream.handle),mode);
  if Fmovie<>nil then
  begin
    Fcanvas := Canvas;
    fbmp := TBitmap.create;
    Fbmp.Width       := fmovie.Width;
    Fbmp.height      := fmovie.height;
    fbmp.PixelFormat := pf32bit;
    fbmp.Canvas.Brush.Color:=0;
    fbmp.Canvas.FillRect(fbmp.Canvas.ClipRect);

  end
  else Fstream.Free;
  inherited create(true);
end;

{-------------------------------------------------------------------------------
    ::Draw
    Scales and blits the current frame to canvas
    notice that the movie is flipped horizontally
-------------------------------------------------------------------------------}
procedure TMovieThread.draw;
var
  R:Trect;
  i:integer;
begin
  R.Left := Fcanvas.ClipRect.Left;
  R.right := Fcanvas.ClipRect.Right;
  R.Top := Fcanvas.ClipRect.bottom;
  R.Bottom := Fcanvas.ClipRect.top;

  Fcanvas.StretchDraw(r,fbmp);
end;

{-------------------------------------------------------------------------------
    ::Execute
    Enters a loop that processes the frames of the binkfile
-------------------------------------------------------------------------------}
procedure TMovieThread.Execute;
var
  B:windows.tbitmap;
begin
  if Fmovie=nil then exit;
  // need this to obtain the address of the bitmap data
  GetObject(Fbmp.Handle,sizeof(bitmap),@b);

  while (not terminated)  do
  if FGoto then
   begin
     BinkGoto(Fmovie,FGotoframe,1);
     Fgoto:=false;
     BinkDoFrame(Fmovie);
   end
  else
   begin
    // Decompress frame
    BinkDoFrame(Fmovie);

    // Copy to buffer
    BinkCopyToBuffer( Fmovie,b.bmBits,
                      Fmovie.width*4,  // Pitch = Width*(bpp/8)
                      $FFFFFFFF,       // ??
                      0,               // x offset
                      0, 	       // Y offset!
                      longword(form1.ComboBox2.Items.Objects
                        [form1.ComboBox2.ItemIndex])    // BufferType
                    );

    // Draw the frame
    Synchronize(draw);
    // Send notification
    Synchronize(Notify);
    // Go to next frame
    while BinkWait(Fmovie)<>0 do; // busy waiting, but who cares when viewing a
                                  // cutscene, right?
    BinkNextFrame(Fmovie);
   end;

  //Release movie  (welease Bwian!)
  FSTream.Free;
  BinkClose(Fmovie);
end;


procedure TMovieThread.Play;
begin
  Resume;
end;


procedure TMovieThread.GotoFrame(Frame: longword);
begin
  FGoto:=true;
  FGotoframe:=Frame;
end;

procedure TMovieThread.Notify;
begin
  BinkGetRealtime(Fmovie,FRealTimeInfo,1);
  if assigned(onNotify) then OnNotify(Fmovie,FRealTimeInfo);

end;

{-------------------------------------------------------------------------------
    TFORM1 Methods :

-------------------------------------------------------------------------------}

procedure TForm1.FormCreate(Sender: TObject);
var sound:string;
begin
  // Start sound system
  if BinkSetSoundSystem(binkOpenDirectSound,0) then
    sound:='Using dsound'
  else if BinkSetSoundSystem(binkOpenWaveOut,0) then
    sound:='Using waveout'
  else
    sound:='No sound';

  //Fill display options
  ComboBox1.Items.AddObject('MOVIE DEFAULT',nil);
  ComboBox1.Items.AddObject('BINK_OPEN_2xHEIGHT_DBL',
                            ptr(BINK_OPEN_2xHEIGHT_DBL));
  ComboBox1.Items.AddObject('BINK_OPEN_2xHEIGHT_INT',
                            ptr(BINK_OPEN_2xHEIGHT_INT));
  ComboBox1.Items.AddObject('BINK_OPEN_2xSIZE',
                            ptr(BINK_OPEN_2xSIZE));
  ComboBox1.Items.AddObject('BINK_OPEN_2xWIDTH_DBL',
                            ptr(BINK_OPEN_2xWIDTH_DBL));
  ComboBox1.Items.AddObject('BINK_OPEN_2xSIZE_INT',
                            ptr(BINK_OPEN_2xSIZE_INT));
  ComboBox1.ItemIndex:=0;


  combobox2.Items.addObject('BINKSURFACE32',ptr(BINKSURFACE32) );
  combobox2.Items.addObject('BINKSURFACE32R',ptr(BINKSURFACE32R) );
  combobox2.Items.addObject('BINKSURFACE32A',ptr(BINKSURFACE32) );
  combobox2.Items.addObject('BINKSURFACE32AR',ptr(BINKSURFACE32AR) );
  ComboBox2.ItemIndex:=0;

  Caption := 'Delphi BinkPlayer - by Lars Peter Christiansen';
  StatusBar1.Panels[0].Text:=
    BinkDll+' Version: '+GetBinkVersion()+' ('+sound+')';

end;


procedure TForm1.PlayButtonClick(Sender: TObject);
var
  Flags:longword;
  s:string;
begin
  FreeAndNil(mt);

  Flags:=flags OR longword(ComboBox1.Items.Objects[ComboBox1.Itemindex]);

  mt := TMovieThread.Create(
           edit1.text,
           PaintBox1.Canvas,
           StrToIntDef(edit2.text,0),
           Flags  );
  mt.OnNotify := threadnotify;
  mt.Priority:=tpLower;
  FillInfo(mt.Fmovie);

  mt.Play;
  s := BinkGetError;
  if s<>'' then showmessage(s);
end;



procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(mt);
end;


procedure TForm1.fillinfo(bnk: hbink);
begin
  if bnk=nil then exit;
  ScrollBar1.Max := bnk.Frames;
end;


procedure TForm1.ThreadNotify;
var F:single;
begin
  ScrollBar1.Position := info.CurrentFrame;
  // frame
  StatusBar1.Panels[1].Text := IntToStr(info.CurrentFrame)+
                               '\'+inttostr(Bink.Frames);
  // rate
  StatusBar1.Panels[2].Text := IntToStr(info.DataRate);
  // stored framerate
  StatusBar1.Panels[3].Text := IntToStr(bink.FrameRate);
  // computed framerate
  if info.MSPrFrame=0 then F:=0 else F := 1000/info.MSPrFrame;
  StatusBar1.Panels[4].Text := FloatToStrF(F,fffixed,7,1 ) ;
end;


procedure TForm1.ScrollBar1Scroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
begin
  if (sender is TScrollbar) and (mt<>nil) then
  mt.GotoFrame(ScrollBar1.Position);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FreeAndNil(mt);
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  BinkSetVolume(mt.Fmovie,(255-trackbar1.position)*(256));
end;

procedure TForm1.TrackBar2Change(Sender: TObject);
begin
  BinkSetPan(mt.Fmovie,256*trackbar2.Position);
end;

end.
